/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

/**
 * @author iwata
 *
 * ̐ꂽRg̑}ev[gύX邽
 * EBhE > ݒ > Java > R[h > R[hƃRg
 */
public class AntColonyJNI implements AntColonyOptimizer {
	private static String libname = "org_agmodel_bioinfo_antmap_AntColonyJNI";
	private AcoParamsBean params;
	private long seed; //JNI痘p
	private double [] dist_1d;
	private int n_city;

	private native double findBestTour(
							double alpha,
							double beta,
							double rho,
							double q,
							int n_iteration,
							int n_ant,
							double init_tau,
							double p_fancy,
							double weight,
							double [] dist_1d,
							int [] order);

	static {
		System.loadLibrary(libname);
	}

	/**
	 * Construct an AntColonyJNI object (JNI version of AntColony)
	 * @param params ACO parameters
	 * @param dist Distance matrix
	 */
	public AntColonyJNI(AcoParamsBean params, double [][] dist) {
		this(params, dist, System.currentTimeMillis());
	}

	/**
	 * Construct an AntColony object (100% pure Java version of AntColony)
	 * @param params ACO parameters
	 * @param dist Distance matrix
	 * @param seed Seed for a random number generator
	 */
	public AntColonyJNI(AcoParamsBean params, double [][] dist, long seed) {
		this.seed = seed;
		this.params = params;
		dist_1d = convertDoubleArray2DTo1D(dist);
		n_city = dist.length;
	}

	/**
	 * Find the best tour by using Ant Colony Optimization
	 * @ return Best tour
	 */
	public TourBean findBestTour() {
		int [] bestOrder = new int [n_city];
		double bestLength = findBestTour(
								params.getAlpha(),
								params.getBeta(),
								params.getRho(),
								params.getQ(),
								params.getN_iteration(),
								params.getN_ant(),
								params.getInit_tau(),
								params.getP_fancy(),
								params.getWeight(),
								dist_1d,
								bestOrder);
		return new TourBean(bestLength, bestOrder);
	}

	private double [] convertDoubleArray2DTo1D(double [][] da_2d) {
		double [] da_1d = new double [da_2d.length * da_2d[0].length];
		for(int i = 0; i < da_2d.length; i++ ) {
			for(int j = 0; j < da_2d[0].length; j++ ) {
				da_1d[i * da_2d[0].length + j] = da_2d[i][j];
			}
		}
		return da_1d;
	}
	
	public long getSeed() {
		return seed;
	}
	
	public void setSeed(long seed) {
		this.seed = seed;
	}
}
