/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmapgui;

import javax.swing.JTextArea;
import javax.swing.JProgressBar;
import org.agmodel.bioinfo.common.StringFormat;
import java.text.DecimalFormat;
import org.agmodel.bioinfo.antmap.*;

/**
 * A thread class for ordering loci by ACO algorithm
 * <p>Title: AntMap</p>
 * <p>Explanation: Linkage mapping software based on ACO algorithm</p>
 * <p>Copyright: Copyright (c) 2004 NARO</p>
 * <p>Company name: NARO</p>
 * @author Hiroyoshi IWATA
 * @version 1.1
 */
public class AntColonyOrdering extends Thread {
  JTextArea log;
  JTextArea result;
  DrawMapPanel map;
  JProgressBar progress;
  MapData [] mapData;
  SegDataBean segDataBean;
  GroupExtractor groupExtractor;
  OrderingOptionBean orderingOptionBean;
  AcoParamsBean acoParamsBean;
  ThreadIndicator threadIndicator;

  public AntColonyOrdering(JTextArea log,
                           JTextArea result,
                           DrawMapPanel map,
                           JProgressBar progress,
                           MapData [] mapData,
                           SegDataBean segDataBean,
                           GroupExtractor groupExtractor,
                           OrderingOptionBean orderingOptionBean,
                           AcoParamsBean acoParamsBean,
                           ThreadIndicator threadIndicator) {
    this.log = log;
    this.result = result;
    this.map = map;
    this.progress = progress;
    this.mapData = mapData;
    this.segDataBean = segDataBean;
    this.groupExtractor = groupExtractor;
    this.orderingOptionBean = orderingOptionBean;
    this.acoParamsBean = acoParamsBean;
    this.threadIndicator = threadIndicator;
  }

  public void run() {
    if(!threadIndicator.isInService()) {
      threadIndicator.setInService(true);
      long start = System.currentTimeMillis();
      orderingMap();
      map.drawMap(mapData);
      long stop = System.currentTimeMillis();
      log.append("    ...ok\n");
      log.append("    run time: " + ((double)(stop - start) / 1000) + " sec\n");
      log.setCaretPosition(log.getText().length());
      progress.setValue(0);
      threadIndicator.setInService(false);
    }
  }

  private void orderingMap() {
    int n_group = groupExtractor.getN_group();
    int groupIndex = orderingOptionBean.getGroupIndex();
    boolean target_all = (groupIndex == -1);
    for(int i = 0; i < n_group; i++ ) {
      if(target_all || (groupIndex == i)) {
        log.append("    Group" + (i + 1) + ": \n");
        log.setCaretPosition(log.getText().length());
        result.append("------------\n");
        result.append("Group" + (i + 1) + ": \n");
        result.setCaretPosition(result.getText().length());

        int[] markerIndex = groupExtractor.getMarkerIndices(i);
        mapData[i] = getBestMap(markerIndex);
        mapData[i].setName("Group" + (i + 1));
        int[] orderedIndex = mapData[i].getMarkerIndexArray();
        log.append("        Best order: ");
        for (int j = 0; j < orderedIndex.length; j++)
          log.append(" " + orderedIndex[j]);
        log.append("\n");
        log.setCaretPosition(log.getText().length());
        DecimalFormat decimalFormat = new DecimalFormat("########0.00");
        StringFormat stringFormat = new StringFormat();
        for (int j = 0; j < mapData[i].getNMarker(); j++) {
          MarkerBean marker = mapData[i].getMarker(j);
          result.append("    ");
          result.append(stringFormat.format(new Integer(marker.getIndex()).
                                            toString(), 5, true));
          result.append(stringFormat.format(marker.getName(), 12, true));
          result.append(stringFormat.format(decimalFormat.format(marker.
              getLocation()), 8, false));
          result.append("\n");
        }
        result.setCaretPosition(result.getText().length());
      }
    }
  }

  private MapData getBestMap(int [] markerIndex) {
    double [][] r = new RMatrixFactory().getR(segDataBean, markerIndex);
    double [][] dist;
    if(orderingOptionBean.getCriterion() == OrderingCriterion.OC_LL) {
      int [][] n = new NMatrixFactory().getN(segDataBean, markerIndex);
      dist = new LDMatrixFactory().convertRN2LD(r, n);
    } else {
      dist = r;
    }
    AntColonyOptimizer aco = new ACOFactory().getAntColonyOptimizer(acoParamsBean, dist);
    TourBean bestTour = new TourBean();
    //int count = 0;
    int maxIteration = orderingOptionBean.getMaxIteration();
    progress.setMinimum(0);
    progress.setMaximum(maxIteration);
    for(int i = 0; i < maxIteration; i++ ) {
      TourBean tempTour = aco.findBestTour();
      log.append("        Tour Length " + (i + 1) + ": " + tempTour.getTourLength() + "\n");
      log.setCaretPosition(log.getText().length());
      //if(bestTour.getTourLength() == tempTour.getTourLength())
      //  count ++;
      //else if(bestTour.getTourLength() > tempTour.getTourLength()) {
      if(bestTour.getTourLength() > tempTour.getTourLength()) {
        bestTour = tempTour;
        //count = 0;
      }
      //if(count == orderingOptionBean.getNConfirm())
      //  break;
      progress.setValue(i + 1);
    }
    progress.setValue(maxIteration);
    new OrderAligner().align(bestTour.getTourOrder(),r);
    int [] orderingIndex = new IndexConverter().getOrderedIndex(markerIndex, bestTour.getTourOrder());
    MapData mapData = new MapData(orderingOptionBean.getMapFunction());
    mapData.setMarkers(
        markerIndex,
        bestTour.getTourOrder(),
        new PartialExtractor().extractParitalArray(segDataBean.getMarkerName(), markerIndex),
        r);
    return mapData;
  }


}
