/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.segregation;

/**
 * @author iwata
 * Map function
 * Calculate map distance from recombinatino rate, or vice verse, on the basis of specific map function.
 * ̐ꂽRg̑}ev[gύX邽
 * EBhE > ݒ > Java > R[h > R[hƃRg
 */
public class MapFunction {
	public static final int MF_HALDANE = 0;
	public static final int MF_KOSAMBI = 1;
	public static final double MAX_DIST = 1000;
	public static final String MF_HALDANE_NAME = "Haldane";
	public static final String MF_KOSAMBI_NAME = "Kosambi";

	private int function = 0;	//Default function is HALDANE

	public MapFunction() {
	}

	public MapFunction(int function) {
		this.function = function;
	}

	public double getRate(double x) {
		double r;
		if(function == MF_KOSAMBI)
			r = getKosambiRate(x);
		else //function == MF_HALDANE (Default)
			r = getHaldaneRate(x);
		return r;
	}

	public static String getFunctionName(int function) {
		if(function == MF_KOSAMBI)
			return MF_KOSAMBI_NAME;
		return MF_HALDANE_NAME;
	}

	public double getDistance(double r) {
		double d;
		if(function == MF_KOSAMBI)
			d = getKosambiDistance(r);
		else //function == MF_HALDANE (Default)
			d = getHaldaneDistance(r);
		return d;
	}

	/**
	 * Calculate recombination rate from map distance on the basis of Holdane's function
	 * @param x Map distance. The unit is Morgan.
	 * @return Recombination rate.
	 */
	public double getHaldaneRate(double x) {
		return 0.5 * (1 - Math.exp(-2 * x));
	}

	/**
	 * Calculate map distance from recombination rate on the basis of Holdane's function
	 * @param r Recombination rate.
	 * @return Map distance. The unit is Morgan.
	 */
	public double getHaldaneDistance(double r) {
		if(r >= 0.5) return MAX_DIST;
		return -0.5 * Math.log(1 - 2 * r);
	}

	/**
	 * Calculate recombination rate from map distance on the basis of Kosambi's function
	 * @param x Map distance. The unit is Morgan.
	 * @return Recombination rate.
	 */
	public double getKosambiRate(double x) {
		return 0.5 * tanh(2 * x);
	}

	/**
	 * Calculate map distance from recombination rate on the basis of Kosambi's function
	 * @param r Recombinatino rate.
	 * @return Map distance. The unit is Morgan.
	 */
	public double getKosambiDistance(double r) {
		if(r >= 0.5) return MAX_DIST;
		return 0.25 * Math.log((1 + 2 * r) / (1 - 2 * r));
	}

	/**
	 * hyperbolic tangent
	 * JavaASYTiZp]_Ёjp
	 * @param x x
	 * @return tanh(x)
	 */
	private double tanh(double x) {
		double EPS5 = 0.001;
		if(x >  EPS5) return 2 / (1 + Math.exp(-2 * x)) - 1;
		if(x < -EPS5) return 1 - 2 / (Math.exp(2 * x) + 1);
		return x * (1 - x * x / 3);
	}

	/**
	 * @return
	 */
	public int getFunction() {
		return function;
	}

	/**
	 * @param i
	 */
	public void setFunction(int i) {
		function = i;
	}

}
