/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.segregation;

import org.agmodel.bioinfo.common.BisectionBean;
import org.agmodel.bioinfo.stat.ChiSquareDist;

/**
 * @author iwata
 *
 */
public class TwoPointAnalysis {
	private static final double MAX_REC = 1.0;

	public double getR_BC(char [] l1, char [] l2) {
		if(l1.length != l2.length)
			return -1.0;

		int a = 0, b = 0;
		for(int i = 0; i < l1.length; i++ ) {
			if((l1[i] == 'A' && l2[i] == 'A') || (l1[i] == 'H' && l2[i] == 'H')) {
				a ++;
			} else if((l1[i] == 'A' && l2[i] == 'H') || (l1[i] == 'H' && l2[i] == 'A')) {
				b ++;
			}
		}
		return (double) b / (a + b);
	}

	public double getR_DH(char [] l1, char [] l2) {
		if(l1.length != l2.length)
			return -1.0;

		int a = 0, b = 0;
		for(int i = 0; i < l1.length; i++ ) {
			if((l1[i] == 'A' && l2[i] == 'A') || (l1[i] == 'B' && l2[i] == 'B')) {
				a ++;
			} else if((l1[i] == 'A' && l2[i] == 'B') || (l1[i] == 'B' && l2[i] == 'A')) {
				b ++;
			}
		}
		return (double) b / (a + b);

	}

	public double getR_RIL(char [] l1, char [] l2) {
		if(l1.length != l2.length)
			return -1.0;

		int a = 0, b = 0;
		for(int i = 0; i < l1.length; i++ ) {
			if((l1[i] == 'A' && l2[i] == 'A') || (l1[i] == 'B' && l2[i] == 'B')) {
				a ++;
			} else if((l1[i] == 'A' && l2[i] == 'B') || (l1[i] == 'B' && l2[i] == 'A')) {
				b ++;
			}
		}
		if(a == 0)
			return 0.5;
		else
			return (double) b / (2 * a);

	}

	public double getR_F2(char [] l1, char [] l2, int markerType1, int markerType2) {
		if(l1.length != l2.length)
			return -1.0;

		int [] a = new int[9];
		if(markerType1 == MarkerType.MT_CODOM && markerType2 == MarkerType.MT_CODOM) { //Codom-Codom
			for(int i = 0; i < l1.length; i++ ) {
				if(l1[i] == 'A' && l2[i] == 'A') {
					a[0] ++;
				} else if(l1[i] == 'A' && l2[i] == 'H') {
					a[1] ++;
				} else if(l1[i] == 'A' && l2[i] == 'B') {
					a[2] ++;
				} else if(l1[i] == 'H' && l2[i] == 'A') {
					a[3] ++;
				} else if(l1[i] == 'H' && l2[i] == 'H') {
					a[4] ++;
				} else if(l1[i] == 'H' && l2[i] == 'B') {
					a[5] ++;
				} else if(l1[i] == 'B' && l2[i] == 'A') {
					a[6] ++;
				} else if(l1[i] == 'B' && l2[i] == 'H') {
					a[7] ++;
				} else if(l1[i] == 'B' && l2[i] == 'B') {
					a[8] ++;
				}
			}
			return BisectionBean.solve(new Sr_CC(a), 0.0, MAX_REC, 0);
		} else if(markerType1 == MarkerType.MT_CODOM && (markerType2 == MarkerType.MT_DOM_P1 || markerType2 == MarkerType.MT_DOM_P2)) { //Codom-Dom
			for(int i = 0; i < l1.length; i++ ) {
				if((l1[i] == 'A' && l2[i] == 'D') || (l1[i] == 'B' && l2[i] == 'C')) {
					a[0] ++;
				} else if((l1[i] == 'A' && l2[i] == 'B') || (l1[i] == 'B' && l2[i] == 'A')) {
					a[1] ++;
				} else if(l1[i] == 'H' && (l2[i] == 'D' || l2[i] == 'C')) {
					a[2] ++;
				} else if(l1[i] == 'H' && (l2[i] == 'B' || l2[i] == 'A')) {
					a[3] ++;
				} else if((l1[i] == 'B' && l2[i] == 'D') || (l1[i] == 'A' && l2[i] == 'C')) {
					a[4] ++;
				} else if((l1[i] == 'B' && l2[i] == 'B') || (l1[i] == 'A' && l2[i] == 'A')) {
					a[5] ++;
				}
			}
			return BisectionBean.solve(new Sr_CD(a), 0.0, MAX_REC, 0);
		} else if((markerType1 == MarkerType.MT_DOM_P1 || markerType1 == MarkerType.MT_DOM_P2) && markerType2 == MarkerType.MT_CODOM) { //Dom-Codom
			for(int i = 0; i < l1.length; i++ ) {
				if((l2[i] == 'A' && l1[i] == 'D') || (l2[i] == 'B' && l1[i] == 'C')) {
					a[0] ++;
				} else if((l2[i] == 'A' && l1[i] == 'B') || (l2[i] == 'B' && l1[i] == 'A')) {
					a[1] ++;
				} else if(l2[i] == 'H' && (l1[i] == 'D' || l1[i] == 'C')) {
					a[2] ++;
				} else if(l2[i] == 'H' && (l1[i] == 'B' || l1[i] == 'A')) {
					a[3] ++;
				} else if((l2[i] == 'B' && l1[i] == 'D') || (l2[i] == 'A' && l1[i] == 'C')) {
					a[4] ++;
				} else if((l2[i] == 'B' && l1[i] == 'B') || (l2[i] == 'A' && l1[i] == 'A')) {
					a[5] ++;
				}
			}
			return BisectionBean.solve(new Sr_CD(a), 0.0, MAX_REC, 0);
		} else if((markerType1 == MarkerType.MT_DOM_P1 && markerType2 == MarkerType.MT_DOM_P1) || (markerType1 == MarkerType.MT_DOM_P2 && markerType2 == MarkerType.MT_DOM_P2)) { //Dom-Dom-Coupling
			for(int i = 0; i < l1.length; i++ ) {
				if((l1[i] == 'D' && l2[i] == 'D') || (l1[i] == 'C' && l2[i] == 'C')) {
					a[0] ++;
				} else if((l1[i] == 'D' && l2[i] == 'B') || (l1[i] == 'C' && l2[i] == 'A')) {
					a[1] ++;
				} else if((l1[i] == 'B' && l2[i] == 'D') || (l1[i] == 'A' && l2[i] == 'C')) {
					a[2] ++;
				} else if((l1[i] == 'B' && l2[i] == 'B') || (l1[i] == 'A' && l2[i] == 'A')) {
					a[3] ++;
				}
			}
			return BisectionBean.solve(new Sr_DD_Cup(a), 0.0, MAX_REC, 0);
		} else if((markerType1 == MarkerType.MT_DOM_P1 && markerType2 == MarkerType.MT_DOM_P2) || (markerType1 == MarkerType.MT_DOM_P2 && markerType2 == MarkerType.MT_DOM_P1)) { //Dom-Dom-Repulsion
			for(int i = 0; i < l1.length; i++ ) {
				if((l1[i] == 'D' && l2[i] == 'C') || (l1[i] == 'C' && l2[i] == 'D')) {
					a[0] ++;
				} else if((l1[i] == 'D' && l2[i] == 'A') || (l1[i] == 'C' && l2[i] == 'B')) {
					a[1] ++;
				} else if((l1[i] == 'B' && l2[i] == 'C') || (l1[i] == 'A' && l2[i] == 'D')) {
					a[2] ++;
				} else if((l1[i] == 'B' && l2[i] == 'A') || (l1[i] == 'A' && l2[i] == 'B')) {
					a[3] ++;
				}
			}
			return BisectionBean.solve(new Sr_DD_Rep(a), 0.0, MAX_REC, 0);
		}
		return -1.0;
	}

	/**
	 * Calculate recombination rate for backcross
	 * @param l1 Array of genotypes for locus1
	 * ('A' (homozygote for the recurrent parent genotype),
	 *  'H' (heterozogote),
	 *  '-' (missing data))
	 * @param l2 Array of genotypes for locus2 (Genotype description is the same as locus1)
	 * @return RecombinationBean.
	 */
	public RecombinationBean getRecombinationBean_BC(char [] l1, char [] l2) {
		RecombinationBean rb = new RecombinationBean();
		if(l1.length != l2.length) {
			rb.setError(true);
			return rb;
		}

		int a = 0, b = 0;
		for(int i = 0; i < l1.length; i++ ) {
			if((l1[i] == 'A' && l2[i] == 'A') || (l1[i] == 'H' && l2[i] == 'H')) {
				a ++;
			} else if((l1[i] == 'A' && l2[i] == 'H') || (l1[i] == 'H' && l2[i] == 'A')) {
				b ++;
			}
		}
		int n = a + b;
		double r = (double) b / n;
		double chi2 = (double) ((a - b) * (a - b)) / n;
		double p = ChiSquareDist.getUpperProb(1, chi2);
		double lod = (b * Math.log(r) + a * Math.log(1 - r) - n * Math.log(0.5)) / Math.log(10);

		rb.setN(n);
		rb.setValue(r);
		rb.setChi2(chi2);
		rb.setP(p);
		rb.setLod(lod);
		rb.setError(false);

		return rb;
	}

	/**
	 * Calculate recombination rate for doubled haploid
	 * @param l1 Array of genotypes for locus1
	 * ('A' (homozygote for parental genotype a),
	 *  'B' (homozygote for paretnal genotype b),
	 *  '-' (missing data))
	 * @param l2 Array of gneotypes for locus2 (Genotype description is the same as locus1)
	 * @return Recombination rate (If array size of l1 and l2 is not identical, return -1).
	 */
	public RecombinationBean getRecombinationBean_DH(char [] l1, char [] l2) {
		RecombinationBean rb = new RecombinationBean();
		if(l1.length != l2.length) {
			rb.setError(true);
			return rb;
		}

		int a = 0, b = 0;
		for(int i = 0; i < l1.length; i++ ) {
			if((l1[i] == 'A' && l2[i] == 'A') || (l1[i] == 'B' && l2[i] == 'B')) {
				a ++;
			} else if((l1[i] == 'A' && l2[i] == 'B') || (l1[i] == 'B' && l2[i] == 'A')) {
				b ++;
			}
		}
		int n = a + b;
		double r = (double) b / n;
		double chi2 = (double) ((a - b) * (a - b)) / n;
		double p = ChiSquareDist.getUpperProb(1, chi2);
		double lod = (b * Math.log(r) + a * Math.log(1 - r) - n * Math.log(0.5)) / Math.log(10);

		rb.setN(n);
		rb.setValue(r);
		rb.setChi2(chi2);
		rb.setP(p);
		rb.setLod(lod);
		rb.setError(false);

		return rb;
	}

	/**
	 * Calculate recombination rate for recombinant inbred line
	 * @param l1 Array of genotypes for locus1
	 * ('A' (homozygote for parental genotype a),
	 *  'B' (homozygote for paretnal genotype b),
	 *  '-' (missing data))
	 * @param l2 Array of gneotypes for locus2 (Genotype description is the same as locus1)
	 * @return Recombination rate (If array size of l1 and l2 is not identical, return -1).
	 */
	public RecombinationBean getRecombinationBean_RIL(char [] l1, char [] l2) {
		RecombinationBean rb = new RecombinationBean();
		if(l1.length != l2.length) {
			rb.setError(true);
			return rb;
		}

		int a = 0, b = 0;
		for(int i = 0; i < l1.length; i++ ) {
			if((l1[i] == 'A' && l2[i] == 'A') || (l1[i] == 'B' && l2[i] == 'B')) {
				a ++;
			} else if((l1[i] == 'A' && l2[i] == 'B') || (l1[i] == 'B' && l2[i] == 'A')) {
				b ++;
			}
		}
		if(a == 0) {
			rb.setError(true);
		} else {
			int n = a + b;
			double r = (double) b / (2 * a);
			double chi2 = (double) ((a - b) * (a - b)) / n;
			double p = ChiSquareDist.getUpperProb(1, chi2);
			double lod = (b * Math.log(r) - n * Math.log(1 + 2 * r) - b * Math.log(0.5) + n * Math.log(2)) / Math.log(10);

			rb.setN(n);
			rb.setValue(r);
			rb.setChi2(chi2);
			rb.setP(p);
			rb.setLod(lod);
			rb.setError(false);
		}

		return rb;
	}

	/**
	 * Calculate recombination rate for F2
	 * @param l1 Array of genotypes for locus1
	 * ('A' (homozygote for parental genotype a),
	 *  'B' (homozygote for paretnal genotype b),
	 *  'H' (heterozygote carrying both alleles a and b),
	 *  'C' (not a homozygote for allele a (either bb or ab genotype)),
	 *  'D' (not a homozygote for allele b (either aa or ab genotype)),
	 *  '-' (missing data))
	 * @param l2 Array of gneotypes for locus2 (Genotype description is the same as locus1)
	 * @return Recombination rate (If array size of l1 and l2 is not identical, return -1).
	 */
	public RecombinationBean getRecombinationBean_F2(char [] l1, char [] l2, int markerType1, int markerType2) {
		RecombinationBean rb = new RecombinationBean();
		if(l1.length != l2.length) {
			rb.setError(true);
			return rb;
		}

		int [] a = new int[9];
		int n = 0;
		double r = 0.0;
		double chi2 = 0.0;
		double p = 1.0;
		double lod = 0.0;

		if(markerType1 == MarkerType.MT_CODOM && markerType2 == MarkerType.MT_CODOM) { //Codom-Codom
			for(int i = 0; i < l1.length; i++ ) {
				if(l1[i] == 'A' && l2[i] == 'A') {
					a[0] ++;
				} else if(l1[i] == 'A' && l2[i] == 'H') {
					a[1] ++;
				} else if(l1[i] == 'A' && l2[i] == 'B') {
					a[2] ++;
				} else if(l1[i] == 'H' && l2[i] == 'A') {
					a[3] ++;
				} else if(l1[i] == 'H' && l2[i] == 'H') {
					a[4] ++;
				} else if(l1[i] == 'H' && l2[i] == 'B') {
					a[5] ++;
				} else if(l1[i] == 'B' && l2[i] == 'A') {
					a[6] ++;
				} else if(l1[i] == 'B' && l2[i] == 'H') {
					a[7] ++;
				} else if(l1[i] == 'B' && l2[i] == 'B') {
					a[8] ++;
				}
			}
			n = sumup(a, 8);
			if(n == 0) {
				rb.setError(true);
				return rb;
			}
			r = BisectionBean.solve(new Sr_CC(a), 0.0, MAX_REC, 0);
			chi2 = calcChi2_CC(a);
			p = ChiSquareDist.getUpperProb(4, chi2);
			lod = ((2 * (a[0] + a[8]) + a[1] + a[3] + a[5] + a[7]) * Math.log(1 - r)
				+ (a[1] + a[3] + a[5] + a[7] + 2 * (a[2] + a[6])) * Math.log(r)
				+ a[4] * Math.log(1 - 2 * r + 2 * r * r)
				- (2 * (a[0] + a[2] + a[6] + a[8]) + a[1] + a[3] + a[4] + a[5] + a[7]) * Math.log(0.5)) / Math.log(10);
		} else if(markerType1 == MarkerType.MT_CODOM && (markerType2 == MarkerType.MT_DOM_P1 || markerType2 == MarkerType.MT_DOM_P2)) { //Codom-Dom
			for(int i = 0; i < l1.length; i++ ) {
				if((l1[i] == 'A' && l2[i] == 'D') || (l1[i] == 'B' && l2[i] == 'C')) {
					a[0] ++;
				} else if((l1[i] == 'A' && l2[i] == 'B') || (l1[i] == 'B' && l2[i] == 'A')) {
					a[1] ++;
				} else if(l1[i] == 'H' && (l2[i] == 'D' || l2[i] == 'C')) {
					a[2] ++;
				} else if(l1[i] == 'H' && (l2[i] == 'B' || l2[i] == 'A')) {
					a[3] ++;
				} else if((l1[i] == 'B' && l2[i] == 'D') || (l1[i] == 'A' && l2[i] == 'C')) {
					a[4] ++;
				} else if((l1[i] == 'B' && l2[i] == 'B') || (l1[i] == 'A' && l2[i] == 'A')) {
					a[5] ++;
				}
			}
			n = sumup(a, 5);
			if(n == 0) {
				rb.setError(true);
				return rb;
			}
			r = BisectionBean.solve(new Sr_CD(a), 0.0, MAX_REC, 0);
			chi2 = calcChi2_CD(a);
			p = ChiSquareDist.getUpperProb(2, chi2);
			lod = ((a[0] + a[3] + 2 * a[5]) * Math.log(1 - r)
				+ a[0] * Math.log(1 + r)
				+ (a[3] + a[4] + 2 * a[1]) * Math.log(r)
				+ a[4] * Math.log(2 - r)
				+ a[2] * Math.log(1 - r + r * r)
				- (a[0] + 2 * a[3] + a[4] + 2 * a[1] + 2 * a[5]) * Math.log(0.5)
				- (a[0] + a[4]) * Math.log(1.5)
				- a[2] * Math.log(0.75)) / Math.log(10);
		} else if((markerType1 == MarkerType.MT_DOM_P1 || markerType1 == MarkerType.MT_DOM_P2) && markerType2 == MarkerType.MT_CODOM) { //Dom-Codom
			for(int i = 0; i < l1.length; i++ ) {
				if((l2[i] == 'A' && l1[i] == 'D') || (l2[i] == 'B' && l1[i] == 'C')) {
					a[0] ++;
				} else if((l2[i] == 'A' && l1[i] == 'B') || (l2[i] == 'B' && l1[i] == 'A')) {
					a[1] ++;
				} else if(l2[i] == 'H' && (l1[i] == 'D' || l1[i] == 'C')) {
					a[2] ++;
				} else if(l2[i] == 'H' && (l1[i] == 'B' || l1[i] == 'A')) {
					a[3] ++;
				} else if((l2[i] == 'B' && l1[i] == 'D') || (l2[i] == 'A' && l1[i] == 'C')) {
					a[4] ++;
				} else if((l2[i] == 'B' && l1[i] == 'B') || (l2[i] == 'A' && l1[i] == 'A')) {
					a[5] ++;
				}
			}
			n = sumup(a, 5);
			if(n == 0) {
				rb.setError(true);
				return rb;
			}
			r = BisectionBean.solve(new Sr_CD(a), 0.0, MAX_REC, 0);
			chi2 = calcChi2_CD(a);
			p = ChiSquareDist.getUpperProb(2, chi2);
			lod = ((a[0] + a[3] + 2 * a[5]) * Math.log(1 - r)
				+ a[0] * Math.log(1 + r)
				+ (a[3] + a[4] + 2 * a[1]) * Math.log(r)
				+ a[4] * Math.log(2 - r)
				+ a[2] * Math.log(1 - r + r * r)
				- (a[0] + 2 * a[3] + a[4] + 2 * a[1] + 2 * a[5]) * Math.log(0.5)
				- (a[0] + a[4]) * Math.log(1.5)
				- a[2] * Math.log(0.75)) / Math.log(10);
		} else if((markerType1 == MarkerType.MT_DOM_P1 && markerType2 == MarkerType.MT_DOM_P1) || (markerType1 == MarkerType.MT_DOM_P2 && markerType2 == MarkerType.MT_DOM_P2)) { //Dom-Dom-Coupling
			for(int i = 0; i < l1.length; i++ ) {
				if((l1[i] == 'D' && l2[i] == 'D') || (l1[i] == 'C' && l2[i] == 'C')) {
					a[0] ++;
				} else if((l1[i] == 'D' && l2[i] == 'B') || (l1[i] == 'C' && l2[i] == 'A')) {
					a[1] ++;
				} else if((l1[i] == 'B' && l2[i] == 'D') || (l1[i] == 'A' && l2[i] == 'C')) {
					a[2] ++;
				} else if((l1[i] == 'B' && l2[i] == 'B') || (l1[i] == 'A' && l2[i] == 'A')) {
					a[3] ++;
				}
			}
			n = sumup(a, 3);
			if(n == 0) {
				rb.setError(true);
				return rb;
			}
			r = BisectionBean.solve(new Sr_DD_Cup(a), 0.0, MAX_REC, 0);
			chi2 = calcChi2_DD(a);
			p = ChiSquareDist.getUpperProb(1, chi2);
			lod = (a[0] * Math.log(3 - 2 * r + r * r)
				+ (a[1] + a[2]) * Math.log(2 * r - r * r)
				+ 2 * a[3] * Math.log(1 - r)
				- a[0] * Math.log(2.25)
				- (a[1] + a[2]) * Math.log(0.75)
				- 2 * a[3] * Math.log(0.5)) / Math.log(10);
		} else if((markerType1 == MarkerType.MT_DOM_P1 && markerType2 == MarkerType.MT_DOM_P2) || (markerType1 == MarkerType.MT_DOM_P2 && markerType2 == MarkerType.MT_DOM_P1)) { //Dom-Dom-Repulsion
			for(int i = 0; i < l1.length; i++ ) {
				if((l1[i] == 'D' && l2[i] == 'C') || (l1[i] == 'C' && l2[i] == 'D')) {
					a[0] ++;
				} else if((l1[i] == 'D' && l2[i] == 'A') || (l1[i] == 'C' && l2[i] == 'B')) {
					a[1] ++;
				} else if((l1[i] == 'B' && l2[i] == 'C') || (l1[i] == 'A' && l2[i] == 'D')) {
					a[2] ++;
				} else if((l1[i] == 'B' && l2[i] == 'A') || (l1[i] == 'A' && l2[i] == 'B')) {
					a[3] ++;
				}
			}
			n = sumup(a, 3);
			if(n == 0) {
				rb.setError(true);
				return rb;
			}
			r = BisectionBean.solve(new Sr_DD_Rep(a), 0.0, MAX_REC, 0);
			chi2 = calcChi2_DD(a);
			p = ChiSquareDist.getUpperProb(1, chi2);
			lod = (a[0] * Math.log(2 + r * r)
				+ (a[1] + a[2]) * Math.log(1 - r * r)
				+ 2 * a[3] * Math.log(r)
				- a[0] + Math.log(2.25)
				- (a[1] + a[2]) * Math.log(0.75)
				- 2 * a[3] * Math.log(0.5)) / Math.log(10);
		}

		rb.setN(n);
		rb.setValue(r);
		rb.setChi2(chi2);
		rb.setP(p);
		rb.setLod(lod);
		rb.setError(false);

		return rb;
	}

	/**
	 * Obtain marker type
	 * @param l Array of genotypes for the locus
	 * @return Marker typei0 - codominant, 1 - dominant (allele a is dominant), 2 - dominant (allele b is dominant))
	 */
	/*
	private int getMarkerType(char [] l) {
		int markerType = 0;
		for(int i = 0; i < l.length; i++ ) {
			if(l[i] == 'H') {
				break;
			} else if(l[i] == 'D') {
				markerType = 1;
				break;
			} else if(l[i] == 'C') {
				markerType = 2;
				break;
			}
		}
		return markerType;
	}
	*/

	private int sumup(int [] a, int to) {
		int sum = 0;
		for(int i = 0; i <= to; i++ ) {
			sum += a[i];
		}
		return sum;
	}

	private double calcChi2_CC(int [] a) {
		int n = sumup(a, 8);
		double chi_t = 4 * (a[4] * a[4]
			+ 2 * (a[1] * a[1] + a[3] * a[3] + a[5] * a[5] + a[7] * a[7])
			+ 4 * (a[0] * a[0] + a[2] * a[2] + a[6] * a[6] + a[8] * a[8])) / n - n;
		double chi_a = 2 * (2 * (a[0] + a[1] + a[2]) * (a[0] + a[1] + a[2])
			+ (a[3] + a[4] + a[5]) * (a[3] + a[4] + a[5])
			+ 2 * (a[6] + a[7] + a[8]) * (a[6] + a[7] + a[8])) / n - n;
		double chi_b = 2 * (2 * (a[0] + a[3] + a[6]) * (a[0] + a[3] + a[6])
			+ (a[1] + a[4] + a[7]) * (a[1] + a[4] + a[7])
			+ 2 * (a[2] + a[5] + a[8]) * (a[2] + a[5] + a[8])) / n - n;
		return chi_t - chi_a - chi_b;
	}

	private double calcChi2_CD(int [] a) {
		int n = sumup(a, 5);
		double chi_t = 8 * (2 * a[0] * a[0] + a[2] * a[2] + 2 * a[4] * a[4]
			+ 6 * a[1] * a[1] + 3 * a[3] * a[3] + 6 * a[5] * a[5]) / (3 * n) - n;
		double chi_a = 4 * ((a[0] + a[2] + a[4]) * (a[0] + a[2] + a[4])
			+ 3 * (a[1] + a[3] + a[5]) * (a[1] + a[3] + a[5])) / (3 * n) - n;
		double chi_b = 2 * (2 * (a[0] + a[1]) * (a[0] + a[1])
			+ (a[2] + a[4]) * (a[2] + a[4])
			+ (2 * (a[3] + a[5]) * (a[3] + a[5]))) / n - n;
		return chi_t - chi_a - chi_b;
	}

	private double calcChi2_DD(int [] a) {
		int n = sumup(a, 3);
		return (a[0] - 3 * a[1] - 3 * a[2] + 9 * a[3]) * (a[0] - 3 * a[1] - 3 * a[2] + 9 * a[3]) / n;
	}
}

class Sr_CC implements BisectionBean.Function {
	private double aa, bb, cc, dd;

	public Sr_CC(int [] a) {
		aa = a[0] + a[8];
		bb = a[1] + a[3] + a[5] + a[7];
		cc = a[2] + a[6];
		dd = a[4];
	}

	public double of(double r) {
		if(r == 0) return Double.MAX_VALUE;
		return - (2 * aa + bb) / (1 - r)
		        + (bb + 2 * cc) / r
		        + (4 * r - 2) * dd / (1 - 2 * r + 2 * r * r);
	}
}

class Sr_CD implements BisectionBean.Function {
	private double aa, bb, cc, dd, ee;

	public Sr_CD(int [] a) {
		aa = a[0] + a[3] + 2 * a[5];
		bb = a[0];
		cc = a[3] + a[4] + 2 * a[1];
		dd = a[4];
		ee = a[2];
	}

	public double of(double r) {
		if(r == 0) return Double.MAX_VALUE;
		return - aa / (1 - r)
		        + bb / (1 + r)
		        + cc / r
		        - dd / (2 - r)
		        + (2 * r - 1) * ee / (1 - r + r * r);
	}
}

class Sr_DD_Cup implements BisectionBean.Function {
	private double aa, bb, cc;

	public Sr_DD_Cup(int [] a) {
		aa = a[0];
		bb = a[1] + a[2];
		cc = a[3];
	}

	public double of(double r) {
		if(r == 0) return Double.MAX_VALUE;
		return - 2 * (1 - r) * aa / (3 - 2 * r + r * r)
		        + 2 * (1 - r) * bb / (r * (2 - r))
		        - 2 * cc / (1 - r);
	}
}

class Sr_DD_Rep implements BisectionBean.Function {
	private double aa, bb, cc;

	public Sr_DD_Rep(int [] a) {
		aa = a[0];
		bb = a[1] + a[2];
		cc = a[3];
	}

	public double of(double r) {
		if(r == 0) return Double.MAX_VALUE;
		return 2 * r * aa / (2 + r * r)
				- 2 * r * bb / (1 - r * r)
				+ 2 * cc / r;
	}
}
